/* -*- c++ -*-
   god.h

   General Operations Director

   perform operations requiring omnipotence in the simulation
   $Id: god.h,v 1.5 2001/01/11 18:21:24 jinyang Exp $
   */

#ifndef __god_h
#define __god_h

#include <stdarg.h>

#include <object.h>
#include <packet.h>
#include <trace.h>
#include <math.h>
#include <random.h>
#include <cmu/node.h>
#include <cmu/hgps/hdr_hgps.h>

struct entry{
    int num;
    nsaddr_t *list;
};


//bellman-ford route entry
struct bf_rt_entry{
    nsaddr_t prev_hop;
    nsaddr_t next_hop;
    int	dist;
};

struct mac_log_entry{
    double tx;
    double rts_tx;
    double rts_timeout;
    double tx_timeout;
    double rx;
    double backoff;
    double useless_backoff;
    double defering;
    double idle;
    double true_idle;
};
class God;

class GodTimer: public Handler {
public:
    GodTimer(God *g, double t): god_(g),interrupt_time_(t) {};
    void handle(Event *e);

    God *god_;
    double interrupt_time_;
};

class God : public Connector {
public:
        God();

        int             command(int argc, const char* const* argv);

        void            recv(Packet *p, Handler *h);
        void            stampPacket(Packet *p);

	//added by Jinyang
	nsaddr_t getNodeDstAway(nsaddr_t,double);
	nsaddr_t	shortestPathNextHop(nsaddr_t src, nsaddr_t dst);
	int	shortestPathLength(nsaddr_t src, nsaddr_t dst);
	bool		isAwake(nsaddr_t index);
	bool		isNeighbor(nsaddr_t src, nsaddr_t dst);
	void	handleTimer();
	void	start_maclog();

        int initialized() {
                return num_nodes && min_hops && target_;
        }

        int             hops(int i, int j);
        static God*     instance() { assert(instance_); return instance_; }

	MobileNode **nodelist; //added by Jinyang -- a reference to all the mobile nodes
	Mac	**maclist; //added by Jinyang -- a reference to all the Mac
	struct mac_log_entry *maclog;

#ifdef IDEAL_ROUTE
	bool		getRouteToLoc(nsaddr_t fromnode,location dstloc, nsaddr_t *route, int maxnodes);
	Quadtree        *root;
#endif
	double *wakeup_list; //added by Jinyang -- take care of when each node has woken up
        int num_nodes;

	GodTimer *god_timer;
	Event god_interrupt_event;
	Topography      *T;



private:
	bf_rt_entry *shortest_path;
        int* min_hops;   // square array of num_nodesXnum_nodes
                         // min_hops[i * num_nodes + j] giving 
			 // minhops between i and j
	double *wakelist; //added by Jinyang -- take care of wake sleep scenarios
        static God*     instance_;
	struct entry *dist_to_nodes;
	int max_dis_dist;
	double maxx,maxy;
	int len_x,len_y;

};

#endif
