# grid.tcl
# ======================================================================
# Default Script Options
# ======================================================================

set opt(rt_port) 255
set opt(cc)      "off"            ;# have god check the caches for bad links?

Agent/GridAgent set sport_ 255
Agent/GridAgent set dport_ 255

# ======================================================================
Class SRNode -superclass MobileNode

SRNode instproc init {args} {
    global opt ns_ tracefd RouterTrace
    $self instvar grid_agent_ dmux_ entry_point_

    eval $self next $args	;# parent class constructor

    # puts "making GridAgent for node [$self id]"
    set grid_agent_ [new Agent/GridAgent]
    $grid_agent_ ip-addr [$self id]
    $grid_agent_ mobile-node $self

    if { $RouterTrace == "ON" } {
	# Recv Target
	set rcvT [cmu-trace Recv "RTR" $self]
	$rcvT target $grid_agent_
	set entry_point_ $rcvT	
    } else {
	# Recv Target
	set entry_point_ $grid_agent_
    }

    #
    # Drop Target (always on regardless of other tracing)
    #
    set drpT [cmu-trace Drop "RTR" $self]
    $grid_agent_ drop-target $drpT

    #
    # Log Target
    #
    set T [new Trace/Generic]
    $T target [$ns_ set nullAgent_]
    $T attach $tracefd
    $T set src_ [$self id]
    $grid_agent_ log-target $T

    $grid_agent_ target $dmux_

    # packets to the Grid port should be dropped, since we've
    # already handled them in the GridAgent at the entry.
    set nullAgent_ [$ns_ set nullAgent_]
    $dmux_ install $opt(rt_port) $nullAgent_

    # SRNodes don't use the IP addr classifier.  The HGPSAgent should
    # be the entry point
    $self instvar classifier_
    set classifier_ "srnode made illegal use of classifier_"

}

SRNode instproc start-grid {} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ start-grid
}

SRNode instproc dump-tables {} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ dump-tables
}

SRNode instproc dump-table-sizes {} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ dump-table-sizes
}

SRNode instproc hello-period {args} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ hello-period $args
}

SRNode instproc trigger-period {args} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ trigger-period $args
}

SRNode instproc loc-advertise-period {args} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ loc-advertise-period $args
}

SRNode instproc expire-period {args} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ expire-period $args
}

SRNode instproc max-hello-size {args} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ max-hello-size $args
}

SRNode instproc use-geo-fwd {args} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ use-geo-fwd $args
}

SRNode instproc max-hops {args} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ max-hops $args
}

SRNode instproc num-good-locs {args} {
    $self instvar grid_agent_
    global opt

    $grid_agent_ num-good-locs $args
}

SRNode instproc no-loc {} {
    # no location capability for this node
    $self instvar grid_agent_
    global opt

    $grid_agent_ no-loc
}

SRNode instproc propagate-locs {args} {
    # do location proxy jazz if don't know loc
    $self instvar grid_agent_
    global opt

    $grid_agent_ propagate-locs $args
}

SRNode instproc proxy-straw-man {} {
    # do straw man location proxy stuff
    $self instvar grid_agent_
    global opt

    $grid_agent_ proxy-straw-man
}

SRNode instproc entry {} {
        $self instvar entry_point_
        return $entry_point_
}



SRNode instproc add-interface {args} {
# args are expected to be of the form
# $chan $prop $tracefd $opt(ll) $opt(mac)
    global ns_ opt RouterTrace

    eval $self next $args

    $self instvar grid_agent_ ll_ mac_ ifq_

    #$grid_agent_ mac-addr [$mac_(0) id]

    
    if { $RouterTrace == "ON" } {
	# Send Target
	set sndT [cmu-trace Send "RTR" $self]
	# bag the link-layer, it will just screw us up with ARPs anyway
	$sndT target $ifq_(0)
	$grid_agent_ add-mac $sndT $ifq_(0) $mac_(0)
	
    } else {
	# Send Target
	$grid_agent_ add-mac $ifq_(0) $mac_(0)
    }

}

SRNode instproc wakeup {waketime} {
   $self instvar grid_agent_
   $grid_agent_ wakeup $waketime
}

SRNode instproc test-query {dst} {
    $self instvar grid_agent_
    # $grid_agent_ test-query $dst
}


SRNode instproc reset args {
    $self instvar grid_agent_
    eval $self next $args

    # $grid_agent_ reset
}

# ======================================================================

proc create-mobile-node { id know_loc use_proxy use_strawman } {
    global ns_ chan prop topo tracefd opt node_
    global chan prop tracefd topo opt
    
    set node_($id) [new SRNode]
    
    set node $node_($id)
    $node random-motion 0		;# disable random motion
    $node topography $topo
    
    # connect up the channel
    $node add-interface $chan $prop $opt(ll) $opt(mac)	\
	    $opt(ifq) $opt(ifqlen) $opt(netif) $opt(ant)
    
    #
    # This Trace Target is used to log changes in direction
    # and velocity for the mobile node and log actions of the DSR agent
    #
    set T [new Trace/Generic]
    $T target [$ns_ set nullAgent_]
    $T attach $tracefd
    $T set src_ $id
    $node log-target $T


    ## assorted protocol paramters, mostly for DSDV
    $node hello-period 2
    $node expire-period 18
    $node trigger-period 1
    $node loc-advertise-period 0.25
    $node max-hello-size 1000 ; # bytes
    $node use-geo-fwd 1

    ## local routing hops to use
    $node max-hops 2 


    if { $use_strawman == 1 && $use_proxy > 0 } {
	puts "ERROR, can't do proxy and strawman together"
	exit
    }

    ## don't ``know'' our location
    if { $know_loc == 0 } {
	$node no-loc
    }

    if { $use_proxy > 0 } {
	## if no-loc enabled; remember and propagate any route
	## entries with good locations that we learn
	$node propagate-locs $use_proxy
	## number of route entries with good location to propagate, if
	## propagate-locs enabled.
	$node num-good-locs 4

    } 
    if { $use_strawman == 1 } {
	## don't do proxy stuff, do simple technique instead
	$node proxy-straw-man
    }


    $ns_ at 0.0 "$node start-grid"
    

#    $ns_ at 49.0 "$node dump-tables"
}

