// gridagent.cc
// Douglas S. J. De Couto
// 28 October 2000

// $Id: gridagent.cc,v 1.14 2001/01/08 20:06:32 decouto Exp $

/* structure stolen from dsragent.{h,cc} and hgpsagent.{h,cc} */


#include <sys/types.h>
#include <netinet/in.h>
#include <algorithm>
#include <cstring>

#include <random.h>
#include <cmu/cmu-trace.h>
#include "gridagent.h"
#include "gridrt.h"

#include "vector.cc"

int GridAgent::max_rt_sz_ = 0;
int GridAgent::max_geo_rt_sz_ = 0;
int GridAgent::max_ignorant_rt_sz_ = 0;


/* TclCl support */
static class GridAgentClass : public TclClass
{
public:
  GridAgentClass() : TclClass("Agent/GridAgent") { }
  TclObject* create(int, const char *const *) {	return new GridAgent(); }
} class_GridAgent;

/* end TclCl support */

GridAgent::GridAgent() : 
  Agent(-1), ip_(0), eth_(0), mac_(0), ifq_(0), 
  real_ifq_(0), logtarget_(0), mobile_node_(0),
  hello_timer_(this), expire_timer_(this), 
  trigger_timer_(this), advertise_loc_timer_(this),
  route_table_(0), max_hello_sz_(1000), use_geo_fwd_(0), max_hops_(20),
  propagate_locs_(0), num_good_locs_(2), proxy_straw_man_(false),
  have_proxy_loc_(false), know_loc_(true)
// don't set packet type because we want to work with multiple Grid
// packets -- so we won't use the Agent base class facilities for
// filling in packet fields -- we will do it manually in the agent.
{
  bind("off_grid_lr_", &off_grid_lr_); 
  bind("off_grid_encap_", &off_grid_encap_);
  bind("off_grid_proxy_", &off_grid_proxy_); 
  bind("off_ll_", &off_ll_);
  bind("off_mac_", &off_mac_);
  bind("off_ip_", &off_ip_);

  // advertise our real location to everyone
  /* can we touch this God's MobileNodes now? NO, do it in start...*/
  // advertise_loc(get_loc());
}

GridAgent::~GridAgent() 
{

}


int
GridAgent::command(int argc, const char* const *argv)
{
  if (argc == 2) { 
    /* single word commands */
    if (strcasecmp(argv[1], "reset") == 0) {
      Terminate();
      return Agent::command(argc, argv);
    }
    if (strcasecmp(argv[1], "start-grid") == 0) {
      start();
      return TCL_OK;
    }
    if (strcasecmp(argv[1], "dump-tables") == 0) {
      assert(route_table_);
      route_table_->dump();
      return TCL_OK;
    }
    if (strcasecmp(argv[1], "dump-table-sizes") == 0) {
      print_table_sizes();
      return TCL_OK;
    }
    if (strcasecmp(argv[1], "no-loc") == 0) {
      know_loc_= false;
      return TCL_OK;
    }
    if (strcasecmp(argv[1], "proxy-straw-man") == 0) {
      assert(propagate_locs_ == 0);
      proxy_straw_man_ = true;
      return TCL_OK;
    }
  }
  
  else if (argc == 3) {
    /* two word commands */
    if (strcasecmp(argv[1], "propagate-locs") == 0) {
      assert(!proxy_straw_man_);
      propagate_locs_ = atoi(argv[2]);
      return TCL_OK;
    }
    if (strcasecmp(argv[1], "expire-period") == 0) {
      expire_timer_.t_ = atof(argv[2]);
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "hello-period") == 0) {
      hello_timer_.t_ = atof(argv[2]);
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "trigger-period") == 0) {
      trigger_timer_.t_ = atof(argv[2]);
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "loc-advertise-period") == 0) {
      advertise_loc_timer_.t_ = atof(argv[2]);
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "max-hello-size") == 0) {
      max_hello_sz_ = atoi(argv[2]);
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "use-geo-fwd") == 0) {
      use_geo_fwd_ = atoi(argv[2]);
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "max-hops") == 0) {
      max_hops_ = atoi(argv[2]);
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "num-good-locs") == 0) {
      num_good_locs_ = atoi(argv[2]);
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "ip-addr") == 0) {
      ip_ = atoi(argv[2]);
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "mac-addr") == 0) {
      eth_ = atoi(argv[2]);
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "log-target") == 0) {
      logtarget_ = (Trace *) TclObject::lookup(argv[2]);
      if (logtarget_ == 0) {
	fprintf(stderr, "GridAgent: %s lookup of %s failed\n", argv[1], argv[2]);
	return TCL_ERROR;
      }
      return TCL_OK;
    }
    else if (strcasecmp(argv[1], "mobile-node") == 0) {
      mobile_node_ = (MobileNode *) TclObject::lookup(argv[2]);;
      if (mobile_node_ == 0) {
	fprintf(stderr, "GridAgent: %s lookup of %s failed\n", argv[1], argv[2]);
	return TCL_ERROR;
      }
      return TCL_OK;
    }
  }
  
  else if (argc == 4) {
    /* three word commands */
    if (strcasecmp(argv[1], "add-mac") == 0) {
      real_ifq_ = ifq_ = (PriQueue *) TclObject::lookup(argv[2]);
      if (ifq_ == 0) {
	fprintf(stderr, "GridAgent: %s lookup of %s failed\n", argv[1], argv[2]);
	return TCL_ERROR;
      }
      mac_ = (Mac802_11 *) TclObject::lookup(argv[3]);
      if (mac_ == 0) {
	fprintf(stderr, "GridAgent: %s lookup of %s failed\n", argv[1], argv[3]);
	return TCL_ERROR;
      }
      return TCL_OK;
    }
  }

  else if (argc == 5) {
    /* four word commands */
    // sometimes we add-mac with a trace object as the acket target;
    // but we still need to explicitly hold on to the IFQ.  this is
    // probably the wrong way to do this but it works.
    if (strcasecmp(argv[1], "add-mac") == 0) {
      ifq_ = (PriQueue *) TclObject::lookup(argv[2]);
      if (ifq_ == 0) {
	fprintf(stderr, "GridAgent: %s lookup of %s failed\n", argv[1], argv[2]);
	return TCL_ERROR;
      }
      real_ifq_ = (PriQueue *) TclObject::lookup(argv[3]);
      if (real_ifq_ == 0) {
	fprintf(stderr, "GridAgent: %s lookup of %s failed\n", argv[1], argv[3]);
	return TCL_ERROR;
      }
      mac_ = (Mac802_11 *) TclObject::lookup(argv[4]);
      if (mac_ == 0) {
	fprintf(stderr, "GridAgent: %s lookup of %s failed\n", argv[1], argv[4]);
	return TCL_ERROR;
      }
      return TCL_OK;
    }
  }
  
#if 0
  printf("passing through command: `");
  for (int i = 0; i < argc; i++)
    printf("%s ", argv[i]);
  printf("'\n");
#endif

  return Agent::command(argc, argv);
}

void
GridAgent::recv(Packet *p, Handler *)
{
  /* handle packets with a MAC destination address of this host, or
     the MAC broadcast addr */
  hdr_ip *iph = (hdr_ip *) p->access (off_ip_);
  hdr_cmn *cmnh = (hdr_cmn *) p->access (off_cmn_);

  if (iph->src_ == ip_ && cmnh->num_forwards_ == 0) {
    /* we are originating */
    if (iph->dst_ == ip_) {
      printf("GridAgent: ignoring loopback data packet\n");
      Packet::free(p);
    }
    originate_packet(p);
  }
  else {
    /* we are receiving a packet */
    receive_packet(p);
  }
}

void
GridAgent::Terminate()
{
  /* cleanup all cached/queued packets */
}



void
GridAgent::start()
{
  unsigned int swap_ip = htonl(ip_);
  unsigned char buf[6];
  memcpy(buf, &swap_ip, 4);
  EtherAddress eth(buf);
  
  route_table_ = new GridRT(expire_timer_.t_, hello_timer_.t_, swap_ip, eth, 
			    max_hello_sz_, max_hops_, this);
  hello_timer_.sched(hello_timer_.t_ + 
		     hello_timer_.t_ * GridHelloTimer::HELLO_JITTER * Random::uniform(1));
  expire_timer_.sched(expire_timer_.t_ + 
		      expire_timer_.t_ * GridExpireTimer::EXPIRE_JITTER * Random::uniform(1));
  trigger_timer_.sched(trigger_timer_.t_ + 
		       trigger_timer_.t_ * GridTriggerTimer::TRIGGER_JITTER * Random::uniform(1));
  advertise_loc_timer_.sched(advertise_loc_timer_.t_ + 
			     advertise_loc_timer_.t_ * GridAdvertiseLocTimer::ADVERTISE_JITTER * Random::uniform(1));

}


void
GridAgent::originate_packet(Packet *p)
{
  hdr_ip *iph = (hdr_ip *) p->access (off_ip_);
  hdr_cmn *cmnh = (hdr_cmn *) p->access (off_cmn_);
  hdr_grid_encap *hge = (hdr_grid_encap *) p->access (off_grid_encap_);
  grid_hdr *gh = &hge->gh_;
  grid_nbr_encap *encaph = &hge->encap_;
  
  trace("S %.9f _%d_ originating pkt %d %d->%d", Scheduler::instance().clock(), ip_, cmnh->uid_, iph->src_, iph->dst_);

  // none of the other protocols seem to add their protocol header's
  // size to packet size?  perhaps it gets done automatically
  // somewhere else?
  cmnh->size() += IP_HDR_LEN + sizeof(hdr_grid_encap);

  /* we will assume that all data packets are implicitly Grid_Encap
     packets, but we won't explicitly set the type to PT_GRID_ENCAP,
     otherwise ns won't log the CBR stats */
  assert(cmnh->ptype_ == PT_CBR);
  cmnh->num_forwards_ = 0;
  iph->sport_ = RT_PORT;
  iph->dport_ = RT_PORT;
  
  gh->ip = htonl(ip_);
  set_src_loc(gh);
  
  encaph->dst_ip = htonl(iph->dst_);
  encaph->dst_loc_good = false;
  encaph->hops_travelled = 0;

  forward_packet(p);
}


void
GridAgent::forward_packet(Packet *p, bool is_rescue)
{
  hdr_cmn *cmnh = (hdr_cmn *) p->access (off_cmn_);
  hdr_grid_encap *hge = (hdr_grid_encap *) p->access (off_grid_encap_);
  grid_hdr *gh = &hge->gh_;
  grid_nbr_encap *encaph = &hge->encap_;

  // is_rescue is set for packets that have been pulled out of the IFQ
  // before ever being sent, due to their next hop being broken.  So
  // we don't want to change the visited nodes info --- it has already
  // been set up for this packet
  if (!is_rescue) {
    int num_visits = count(hge->prev_hops_, ip_);
    if (num_visits > 0) 
      trace("S %.9f _%d_ looping pkt %d (%d visits)", Scheduler::instance().clock(), ip_, cmnh->uid_, num_visits);
  }

  // hook up tx failure callback
  cmnh->xmit_failure_ = tx_failed_callback;
  cmnh->xmit_failure_data_ = (void *) this;

  // rule about IP addresses: all the grid headers and algorithms use
  // network byte order, but the ns headers (hdr_ip, hdr_cmn, ip_ in
  // GridAgent, etc.) use host byte order.  remember to convert
  // between them!

  IPAddress next_hop;
  assert(route_table_);
  // check for explicit route
  bool found_next_hop = route_table_->get_next_hop(encaph->dst_ip, &next_hop);
  if (!found_next_hop && use_geo_fwd_ == 1) {
    // check for potential geographic route
    found_next_hop = route_table_->get_next_geo_hop(encaph->dst_ip, &next_hop);
  }
  if (found_next_hop) {
    if (!is_rescue) {
      encaph->hops_travelled++;
      hge->prev_hops_.push_back(ip_);
    }

    gh->tx_ip = htonl(ip_);
    set_tx_loc(gh);
    
    cmnh->next_hop_ = ntohl(next_hop); 
    cmnh->addr_type_ = AF_INET;

    mac_->hdr_dst(p->access(off_mac_), ntohl(next_hop));
    mac_->hdr_src(p->access(off_mac_), ip_);

    Scheduler::instance().schedule(ifq_, p, 0);
  }
  else
    drop(p, DROP_RTR_NO_ROUTE);
}


void 
GridAgent::set_tx_loc(grid_hdr *gh)
{
  if (!know_loc_) {
    gh->tx_loc_good = false;
    gh->tx_loc_err = 0xFFff;
    return;
  }
  gh->tx_loc = get_loc();
  gh->tx_loc_err = 0;
  gh->tx_loc_good = true;
  assert(mobile_node_);
  gh->tx_loc_seq_no = mobile_node_->loc_seq_no_;
}

void 
GridAgent::set_src_loc(grid_hdr *gh)
{
  if (!know_loc_) {
    gh->loc_good = false;
    gh->loc_err = 0xFFff;
    return;
  }
  gh->loc = get_loc();
  gh->loc_err = 0;
  gh->loc_good = true;
  assert(mobile_node_);
  gh->loc_seq_no = mobile_node_->loc_seq_no_;
}


void
GridAgent::tx_failed_callback(Packet *p, void *thunk)
{
  GridAgent *a = (GridAgent *) thunk;
  a->handle_tx_failed(p);
}

void
GridAgent::handle_tx_failed(Packet *p)
{
  // drop(p, DROP_RTR_NO_ROUTE);
  
  hdr_cmn *cmnh = (hdr_cmn *) p->access(off_cmn_);
  assert(cmnh->ptype_ == PT_CBR);
  hdr_ip *iph = (hdr_ip *) p->access(off_ip_);

  IPAddress broken_next_hop = htonl(cmnh->next_hop_);
  
  trace("S %.9f _%d_ rescuing pkt %d %d->%d (next_hop %s)", Scheduler::instance().clock(), ip_, cmnh->uid_, iph->src_, iph->dst_, broken_next_hop.s().cc());

  // fix up the routing table to not use this next hop anymore
  route_table_->bad_next_hop(broken_next_hop);
  
  forward_packet(p, true);
  // resend other packets that were supposed to go through the broken next hop
  Vector<Packet *> other_pkts = extract_pkts_from_ifq(broken_next_hop);
  for (int i = 0; i < other_pkts.size(); i++) {
    hdr_cmn *cmnh2 = (hdr_cmn *) other_pkts[i]->access(off_cmn_);
    assert(cmnh2->ptype_ == PT_CBR);
    forward_packet(other_pkts[i], true);
  }
}

Vector<Packet *>
GridAgent::extract_pkts_from_ifq(IPAddress next_hop_ip)
{
  Vector<Packet *> retval;
  Packet *p = 0;
  while ((p = real_ifq_->prq_get_nexthop(ntohl(next_hop_ip))) != 0)
    retval.push_back(p);
    
  return retval;
}

grid_location
GridAgent::get_loc()
{
  return get_loc(ip_);
}

grid_location
GridAgent::get_loc(int ip)
{
  double x, y, z;
  God::instance()->nodelist[ip]->getLoc(&x, &y, &z);
  return grid_location(x, y);  
}

bool
GridAgent::get_advertised_loc(grid_location *loc, double *when)
{
  return get_advertised_loc(loc, when, ip_);
}

bool
GridAgent::get_advertised_loc(grid_location *loc, double *when, int ip)
{
  if (God::instance()->nodelist[ip]->have_advertised_loc) {
    loc->_x = God::instance()->nodelist[ip]->advertised_x;
    loc->_y = God::instance()->nodelist[ip]->advertised_y;
    *when = God::instance()->nodelist[ip]->last_advertised_jiff;
    return true;
  }
  else 
    return false;
}

void
GridAgent::advertise_loc(grid_location l) 
{
  God::instance()->nodelist[ip_]->have_advertised_loc = true;
  God::instance()->nodelist[ip_]->advertised_x = l._x;
  God::instance()->nodelist[ip_]->advertised_y = l._y;
  God::instance()->nodelist[ip_]->last_advertised_jiff = Scheduler::instance().clock();  
}

void
GridAgent::withdraw_loc()
{
  God::instance()->nodelist[ip_]->have_advertised_loc = false;
}


void
GridAgent::receive_packet(Packet *p)
{
  double jiff = Scheduler::instance().clock();

  hdr_cmn *cmnh = (hdr_cmn *) p->access(off_cmn_);
  hdr_grid_lr *lrh = 0;
  hdr_grid_encap *encaph = 0;
  hdr_grid_proxy *proxyh = 0;

  switch (cmnh->ptype_) {

  case PT_GRID_LR:
    lrh = (hdr_grid_lr *) p->access(off_grid_lr_);
    route_table_->process_grid_packet(p, &lrh->gh_, jiff);
    route_table_->process_route_updates(p, jiff);
    Packet::free(p);
    break;

  case PT_GRID_PROXY:
    assert(0); // not implemented 
    proxyh = (hdr_grid_proxy *) p->access(off_grid_proxy_);
    route_table_->process_grid_packet(p, &proxyh->gh_, jiff);
    route_table_->process_proxy_updates(p, jiff);
    Packet::free(p);
    break;

  case PT_CBR: // assume CBR packets are actually Grid_Encap packets
    encaph = (hdr_grid_encap *) p->access(off_grid_encap_);
    route_table_->process_grid_packet(p, &encaph->gh_, jiff);
    handle_encap_packet(p);
    break;

  default:
    assert(0); // unknown packet type!
  }
}


void
GridAgent::handle_encap_packet(Packet *p)
{
  hdr_ip *iph = (hdr_ip *) p->access(off_ip_);
  hdr_cmn *cmnh = (hdr_cmn *) p->access(off_cmn_);

  if (iph->dst_ == ip_) {
    cmnh->size_ -= (sizeof(hdr_grid_encap) + IP_HDR_LEN);
    target_->recv(p, (Handler *) 0);
  }
  else
    forward_packet(p);
}


void
GridAgent::trace(char* fmt, ...)
{
  va_list ap;
  
  if (!logtarget_) 
    return;

  va_start(ap, fmt);
  vsprintf(logtarget_->buffer(), fmt, ap);
  logtarget_->dump();
  va_end(ap);
}


Packet *
GridAgent::alloc_pkt(int sz)
{
  Packet *p = Packet::alloc(sz);
  struct hdr_cmn *cmnh = (struct hdr_cmn *) p->access(off_cmn_);
  cmnh->uid_ = uidcnt_;
  uidcnt_++;
  return p;
}

void
GridAgent::hello_timer()
{
  assert(route_table_);
  route_table_->run_hello_hook();
}

void
GridAgent::expire_timer()
{
  assert(route_table_);
  route_table_->run_expire_hook();
}

void
GridAgent::trigger_timer()
{
  assert(route_table_);
  route_table_->run_trigger_hook();
}


void
GridAgent::advertise_loc_timer()
{
  if (know_loc_) 
    advertise_loc(get_loc());
  else if (propagate_locs_ && have_proxy_loc_) {
    GridRT::far_entry fe = route_table_->rt_get(proxy_ip_);
    assert(fe.nbr.loc_good);
    advertise_loc(fe.nbr.loc);
  }
  else if (proxy_straw_man_) {
    grid_location loc;
    bool found_loc = route_table_->get_strawman_proxy_loc(&loc);
    if (found_loc)
      advertise_loc(loc);
  }
  // don't withdraw last advertised loc, even if we don't have a new
  // proxy loc or strawman loc -- the packet may still get to us
  // somehow...
}


void
GridAgent::print_table_sizes()
{
  // since static, only one node needs to do it
  if (ip_ != 1)
    return;

  trace("S %.9f _%d_ max rt size %d", Scheduler::instance().clock(), ip_, max_rt_sz_);
  trace("S %.9f _%d_ max geo rt size %d", Scheduler::instance().clock(), ip_, max_geo_rt_sz_);
  trace("S %.9f _%d_ max ignorant rt size %d", Scheduler::instance().clock(), ip_, max_ignorant_rt_sz_);
}


int 
GridAgent::count(Vector<int> v, int val)
{
  int ret = 0;
  for (int i = 0; i < v.size(); i++)
    ret += (v[i] == val ? 1 : 0);
  return ret;
}

void
GridHelloTimer::expire(Event *)
{
  agent_->hello_timer();
  resched(t_ + HELLO_JITTER * t_ * Random::uniform(1)); 
}

void
GridExpireTimer::expire(Event *)
{
  agent_->expire_timer();
  resched(t_ + EXPIRE_JITTER * t_ * Random::uniform(1)); 
}

void
GridTriggerTimer::expire(Event *)
{
  agent_->trigger_timer();
  resched(t_ + TRIGGER_JITTER * t_ * Random::uniform(1)); 
}

void
GridAdvertiseLocTimer::expire(Event *)
{
  agent_->advertise_loc_timer();
  resched(t_ + ADVERTISE_JITTER * t_ * Random::uniform(1)); 
}







