// -*- c++ -*-
// gridagent.h
// Douglas S. J. De Couto
// 28 October 2000

// $Id: gridrt.h,v 1.12 2001/01/08 20:06:33 decouto Exp $

#ifndef _GridRT_h
#define _GridRT_h

#include <packet.h>
#include "hdr_grid.h"
#include "etheraddress.h"
#include "ipaddress.h"
#include "gridagent.h"

#include "vector.hh"
#include "bighashmap.hh"

// routing tables

// most of this code is stolen from click/elements/grid/updateroutes.{hh,cc}
// 8 Jan 2000 -- no longer true, i have extensively modified and
// neatened this up! - decouto

class GridRT {
  friend class GridAgent;

  /* jiffies: in this NS implementation, they are just seconds. */

  class NbrEntry {
    bool _init;
  public:
    EtherAddress eth;
    IPAddress ip;
    double last_updated_jiffies;
    NbrEntry() : _init(false), last_updated_jiffies(-1) { }
    NbrEntry(EtherAddress eth_in, IPAddress ip_in, double jiff) 
      : _init(true), eth(eth_in), ip(ip_in), last_updated_jiffies(jiff) { }
    operator bool() const { return _init; }
    String s() const 
    { return eth.s() + " -- " + ip.s(); }
  };

  typedef BigHashMap<IPAddress, NbrEntry> Table;
  Table _addresses; // immediate nbrs
  /* 
   * _addresses is a mapping from IP to ether for nodes within our
   * radio range.  this information is extracted by snooping on all
   * packets with grid headers .  
   */
 
  
  /* the age field in the grid_nbr_entry is only actually modified
     when new data is put in (that data will contain a new age) ---
     thus the age is the age at last_updated_jiffies --- when the
     entry is copied into a route advertisement, the age in the
     advertisement is decremented appropriately */
  struct far_entry {
    double last_updated_jiffies;
    union {
      double last_advertised;
      double when_triggered;
    };
    grid_nbr_entry nbr;
    far_entry() : last_updated_jiffies(0), last_advertised(-1) { }
    far_entry(double j, grid_nbr_entry n) : last_updated_jiffies(j), last_advertised(-1), nbr(n)  { }
    bool operator<(const far_entry &f) const { return last_advertised < f.last_advertised; }
    bool operator<=(const far_entry &f) const { return last_advertised <= f.last_advertised; }
  };

  typedef BigHashMap<IPAddress, far_entry> FarTable;
  FarTable _rtes; // immediate and multihop nbrs

  static double decr_age(double age, double decr)
  { return (age > decr ? age - decr : 0); }

  double _timeout_jiffies; // how old entries can be
  double _hello_jiffies; // how often hello timer runs

  bool _verbose;

  IPAddress _ipaddr;
  EtherAddress _ethaddr;
  unsigned char _max_hops;
  unsigned int _seq_no;

  int _max_hello_entries;

  GridAgent *_agent;
 
  // can't send more than _max_hello_entries ! 
  void send_routing_update(Vector<grid_nbr_entry> &rte_info);

  /* When we expire routes in the run_expire_hook() or
   run_hello_hook(), we may not have enough room in the transmitted
   routing advertisement to list all the expired routes.  The expired
   routes that don't make it into the packet go into
   _leftover_expired.  expire_routes will empty _leftover_expired and
   merge its old contents with any newly expired entries, and return
   them all in a vector.  (XXX is it possible for _leftover_expired to
   grow to be huge, and some expired routes never get to tbe sent?) */
  Vector<grid_nbr_entry> _leftover_expired;

  /* this vector contains any good route updates that should be sent
     at the next triggered update interval -- it is cleared by the
     hello hook, as that transmits all the good routes anyway */
  Vector<far_entry> _triggered_good_routes;
  /* this vector contains any broken route ads that should be sent at
     the next triggered update -- what should the expire hook do with
     these?  i will ignore it for now XXX */
  Vector<far_entry> _triggered_broken_routes;

  Vector<grid_nbr_entry> expire_routes();
  void merge_triggered_routes(Vector<far_entry> &good_routes, Vector<far_entry> &broken_routes,
			      Vector<IPAddress> &added_routes);

  void pick_new_loc_proxy();
  void check() { assert(!(_agent->proxy_straw_man_ && _agent->propagate_locs_ > 0)); } 
  bool do_proxy_stuff()  { check(); return !_agent->know_loc_ && _agent->propagate_locs_ > 0;  }
  bool do_proxy_stuff1() { check(); return !_agent->know_loc_ && _agent->propagate_locs_ == 1; }
  bool do_proxy_stuff2() { check(); return !_agent->know_loc_ && _agent->propagate_locs_ == 2; }
  bool do_proxy_stuff_with_loc() { check(); return _agent->know_loc_ && _agent->propagate_locs_; }
  bool &have_proxy() { check(); return _agent->have_proxy_loc_; }
  class IPAddress &proxy_ip() { check(); return _agent->proxy_ip_; }
  bool know_loc() { return _agent->know_loc_; }
  
  bool do_strawman_stuff() { check(); return _agent->proxy_straw_man_; }

  struct grid_location get_loc() { return _agent->get_loc(); }
  bool get_advertised_loc(grid_location *loc, double *when, int ip) \
  { return _agent->get_advertised_loc(loc, when, ip); }

  bool rt_contains(IPAddress ip) { return _rtes.findp(ip) != 0; } 
  void rt_insert(far_entry fe); 
  struct far_entry &rt_get_ref(IPAddress ip); // asserts(ip in table)
  struct far_entry rt_get(IPAddress ip); // asserts(ip in table)
  void rt_erase(IPAddress ip); // asserts(ip in table);

  int find_ip(Vector<grid_nbr_entry> v, IPAddress ip);
  int find_ip(Vector<far_entry> v, IPAddress ip);
  
  void qsort(Vector<far_entry> v, int p, int r);
  int partition(Vector<far_entry> v, int p, int r);
  void swap(far_entry &f1, far_entry &f2);
  void sort(Vector<far_entry> v) { qsort(v, 0, v.size() - 1); } 
  int min(int a, int b) { return (a < b) ? a : b; }
  int max(int a, int b) { return (a > b) ? a : b; } 


public:
  GridRT(double timeout, double hello, IPAddress ip, EtherAddress eth, int max_hello_sz,
	 int hops, GridAgent *a, bool verbose = false) :
    _timeout_jiffies(timeout), _hello_jiffies(hello), _verbose(verbose), 
    _ipaddr(ip), _ethaddr(eth), _max_hops(hops), _seq_no(0), 
    _max_hello_entries(max_hello_sz / sizeof(grid_nbr_entry)),
    _agent(a), _leftover_expired(0) { }

  bool get_next_hop(IPAddress ip, IPAddress *next_hop);
  bool get_next_geo_hop(IPAddress ip, IPAddress *next_hop);

  void process_grid_packet(Packet *p, grid_hdr *gh, double jiff); 
  void process_route_updates(Packet *p, double jiff);
  void process_proxy_updates(Packet *p, double jiff);
  void run_hello_hook();
  void run_expire_hook();
  void run_trigger_hook(); 
  void run_proxy_hook();

  void bad_next_hop(IPAddress bad_ip);

  bool get_strawman_proxy_loc(grid_location *loc);

  void dump(); // print out routing tables
};



#endif












