// -*- c++ -*-
// hdr_grid.h
// Douglas S. J. De Couto
// 15 November 2000

// $Id: hdr_grid.h,v 1.8 2001/01/08 20:06:33 decouto Exp $

#ifndef HDR_GRID_H
#define HDR_GRID_H

#include <cstdio>

#include "vector.hh"
#include "string.hh"

/* various grid headers stolen and modified from
   click/elements/grid/grid.hh, click 1.1 16 nov 2000 */

/* see the click code for protocol details */

struct grid_location {
  /* bagged the lat/lon stuff in favor of the ns flat world view. */

  double _x;
  double _y;
  grid_location() : _x(0), _y(0) { };
  grid_location(double x, double y) : _x(x), _y(y) { }

  static double dist(grid_location l1, grid_location l2); // distance, in metres

  String s() const {
    char buf[255];
    snprintf(buf, 255, "%f %f", _x, _y);
    return String(buf);
  }
};


struct grid_hdr {
  // use ns type field

  unsigned int ip;          // Sender's IP address. 
  struct grid_location loc; // Sender's location, set by FixSrcLoc.
  unsigned short loc_err;   // Error radius of position, in metres.  
  bool loc_good;            // If false, don't believe loc  
  unsigned int loc_seq_no;  

  unsigned int tx_ip;       // Transmitter 
  struct grid_location tx_loc;
  unsigned short tx_loc_err;
  bool tx_loc_good;
  unsigned int tx_loc_seq_no;
};

struct grid_nbr_entry {
  unsigned int ip; 
  unsigned int next_hop_ip; // not meaningful in broken route advertisement
  unsigned char max_hops;
  unsigned char num_hops; 
  /* 0 for num_hops indicate that this dest. is unreachable.  if so,
     loc fields are meaningless */
  struct grid_location loc;
  unsigned short loc_err;
  bool loc_good;
  unsigned int seq_no;
  double age; 

  grid_nbr_entry() : ip(0), next_hop_ip(0), max_hops(0), num_hops(0), loc(0, 0), seq_no(0) { }

  grid_nbr_entry(unsigned int _ip, unsigned int _nhip, unsigned char h, unsigned int s)
    : ip(_ip), next_hop_ip(_nhip), num_hops(h), loc(0, 0), seq_no(s) { } 
};

struct grid_hello {
  unsigned int seq_no;
  double age; // decreasing, approximately in secs
  unsigned char max_hops; // if > 0, then propagate for max_hops hops
  unsigned char num_nbrs;
  // for GRID_LR_HELLO packets, packet data is num_nbrs grid_nbr_entry
  // structs.

  grid_hello() : max_hops(0), num_nbrs(0) { }

  static const double MIN_AGE_DECREMENT = 0.01;
  static const double MAX_AGE_DEFAULT = 30; // ~ 30 secs
};

struct grid_nbr_encap {
  unsigned int dst_ip;
  struct grid_location dst_loc;
  unsigned short dst_loc_err;
  bool dst_loc_good;
  unsigned char hops_travelled;
};


struct hdr_grid_lr {
  struct grid_hdr gh_;

  struct grid_hello hello_; // plus nbr info in data

  static int offset_;
};


struct hdr_grid_encap {
  struct grid_hdr gh_;
  
  struct grid_nbr_encap encap_;
  Vector<int> prev_hops_;

  static int offset_;
};


struct proxy_nbr_entry {
  unsigned int max_hops;
  struct grid_nbr_entry nbr;
};

struct hdr_grid_proxy {
  struct grid_hdr gh_;
  unsigned char num_entries;

  // followed by num_entries proxy_nbr_entry structs

  static int offset_;
};


#endif

