#!/usr/bin/perl

$name = $ARGV[0];

$time     = 40;
$mobility = 0;
$route    = 15;

$nodes  = 100;
$period = 30;
$max_x  = 1250;
$max_y  = 1250;

$range  = 250;
$bw     = 2000000;
$max_speed = 10;

$fn = "$max_x-$max_y-n$nodes-$name-t$time-m$mobility-r$route";
open(F0, ">$fn.scen") || die "$fn.scen: $!";

$nn = $nodes+$route+$route;
$mx = $max_x + 250 + 250;
$my = $max_y + 250;

print F0<<EOF
#
# nodes: $nn, max time: $time.00, max x: $mx.00, max y: $my.00
# nominal range: $range link bw: $bw
# pause: 0.00, max speed: $max_speed

EOF
;

# initial positions

for (my $i = 1; $i <= $nodes; $i++) {
  my $pos_x = 250 + rand $max_x;
  my $pos_y = 250 + rand $max_y;
  my $pos_z = 0;
  print F0 "\$node_($i) set X_ $pos_x\n";
  print F0 "\$node_($i) set Y_ $pos_y\n";
  print F0 "\$node_($i) set Z_ $pos_z\n";
}

for (my $i = $nodes+1; $i <= $nodes+$route; $i++) {
  my $pos_x = 100 + rand 100;
  my $pos_y = 250 + rand $max_y;
  my $pos_z = 0;
  print F0 "\$node_($i) set X_ $pos_x\n";
  print F0 "\$node_($i) set Y_ $pos_y\n";
  print F0 "\$node_($i) set Z_ $pos_z\n";
}

for (my $i = $nodes+$route+1; $i <= $nodes+$route+$route; $i++) {
  my $pos_x = 250 + $max_x + 50 + rand 100;
  my $pos_y = 250 + rand $max_y;
  my $pos_z = 0;
  print F0 "\$node_($i) set X_ $pos_x\n";
  print F0 "\$node_($i) set Y_ $pos_y\n";
  print F0 "\$node_($i) set Z_ $pos_z\n";
}

# route packets

if ($route > 0) {
  my $t = $time-30;
  print F0 "set Pkts [expr $t / \$opt(pi)]\n";
  for (my $i = $nodes+1; $i <= $nodes+$route; $i++) { 
    my $dst = $i+$route;
    print F0 "set cbr_($i) [\$ns_ create-connection CBR \$node_($i) CBR \$node_($dst) 0]\n";
    print F0 "\$cbr_($i) set packetSize_ \$opt(ps)\n";
    print F0 "\$cbr_($i) set interval_ \$opt(pi)\n";
    print F0 "\$cbr_($i) set random_ 1\n";
    print F0 "\$cbr_($i) set maxpkts_ \$Pkts\n";
    my $st = 20 + rand(5);
    print F0 "\$ns_ at $st \"\$cbr_($i) start\"\n";
  }
}

# dump tables

$dumps = ($time/$period)-1;

for (my $j = 0; $j < $dumps; $j++) {
  $dt = $j*$period+$period;
  for (my $i = 1; $i <= $nodes+$route+$route; $i++) {
    if ($i == 1) {
      my $dtt = $dt-1;
      print F0 "\$ns_ at $dtt \"\$node_($i) dump-tables\"\n";
    } else {
      print F0 "\$ns_ at $dt \"\$node_($i) dump-tables\"\n";
    }
  }
}
for (my $i = 1; $i <= $nodes+$route+$route; $i++) {
  my $t1 = $time-2;
  my $t2 = $time-1;
  if ($i == 1) {
    print F0 "\$ns_ at $t1 \"\$node_($i) dump-tables\"\n";
  } else {
    print F0 "\$ns_ at $t2 \"\$node_($i) dump-tables\"\n";
  }
}

# mobility

if ($mobility) {
  my %movetime;
  for (my $i = 1; $i <= $nodes; $i++) { $movetime->[$i] = 0; }
  my $not_done = 1;
  while ($not_done == 1) {
    $not_done = 0;
    for (my $i = 1; $i <= $nodes; $i++) {
      if ($movetime->[$i] < $time) {
        my $pos_x = rand $max_x;
        my $pos_y = rand $max_y;
        my $speed = rand $max_speed;
        print F0 "\$ns_ at $movetime->[$i] \"\$node_($i) setdest $pos_x $pos_y $speed\"\n";
        my $next = 20 + rand 30;
        $movetime->[$i] += $next;
	$not_done = 1;
      } 
    }
  }
}

close F0;

