# span.tcl
# ======================================================================
# Default Script Options
# ======================================================================

set opt(rt_port) 255
set opt(cc)      "off"            ;# have god check the caches for bad links?

Agent/SPANAgent set sport_ 255
Agent/SPANAgent set dport_ 255

# ======================================================================
Class SPANNode -superclass MobileNode

SPANNode instproc init {args} {
    global opt ns_ tracefd RouterTrace
    $self instvar span_agent_ dmux_ entry_point_

    eval $self next $args	;# parent class constructor

    # puts "making SPANAgent for node [$self id]"
    set span_agent_ [new Agent/SPANAgent]
    $span_agent_ ip-addr [$self id]
    $span_agent_ mobile-node $self

    if { $RouterTrace == "ON" } {
	# Recv Target
	set rcvT [cmu-trace Recv "RTR" $self]
	$rcvT target $span_agent_
	set entry_point_ $rcvT	
    } else {
	# Recv Target
	set entry_point_ $span_agent_
    }

    #
    # Drop Target (always on regardless of other tracing)
    #
    set drpT [cmu-trace Drop "RTR" $self]
    $span_agent_ drop-target $drpT

    #
    # Log Target
    #
    set T [new Trace/Generic]
    $T target [$ns_ set nullAgent_]
    $T attach $tracefd
    $T set src_ [$self id]
    $span_agent_ log-target $T

    $span_agent_ target $dmux_

    # packets to the SPAN port should be dropped, since we've
    # already handled them in the SPANAgent at the entry.
    set nullAgent_ [$ns_ set nullAgent_]
    $dmux_ install $opt(rt_port) $nullAgent_

    # SPANNodes don't use the IP addr classifier. The SPANAgent should be the
    # entry point

    $self instvar classifier_
    set classifier_ "srnode made illegal use of classifier_"

}

SPANNode instproc start-span {} {
    $self instvar span_agent_
    global opt
    $span_agent_ start-span
}

SPANNode instproc dump-tables {} {
    $self instvar span_agent_
    global opt
    $span_agent_ dump-tables
}

SPANNode instproc drained-energy {drained} {
   $self instvar span_agent_
   $span_agent_ drained-energy $drained
}

SPANNode instproc test-gf {dst} {
    $self instvar span_agent_
    $span_agent_ test-gf $dst
}

SPANNode instproc entry {} { 
    $self instvar entry_point_ 
    return $entry_point_ 
}

SPANNode instproc add-interface {args} {
# args are expected to be of the form
# $chan $prop $tracefd $opt(ll) $opt(mac)

    global ns_ opt RouterTrace

    eval $self next $args

    $self instvar span_agent_ ll_ mac_ ifq_

    if { $RouterTrace == "ON" } {
        # Send Target
          set sndT [cmu-trace Send "RTR" $self]
        # $sndT target $ll_(0)
	# $span_agent_ add-ll $sndT $ifq_(0)
        $sndT target $ifq_(0)
	$span_agent_ add-mac $sndT $mac_(0)
    } else {
        # Send Target
        # $span_agent_ add-ll $ll_(0) $ifq_(0)
        $span_agent_ add-mac $ifq_(0) $mac_(0)
    }
}

# ======================================================================

proc create-mobile-node { id } {
    global ns_ chan prop topo tracefd opt node_
    global chan prop tracefd topo opt
    
    set node_($id) [new SPANNode]
    
    set node $node_($id)
    $node random-motion 0		;# disable random motion
    $node topography $topo
    
    # connect up the channel
    $node add-interface $chan $prop $opt(ll) $opt(mac)	\
	    $opt(ifq) $opt(ifqlen) $opt(netif) $opt(ant)
    
    #
    # This Trace Target is used to log changes in direction
    # and velocity for the mobile node and log actions of the DSR agent
    #
    set T [new Trace/Generic]
    $T target [$ns_ set nullAgent_]
    $T attach $tracefd
    $T set src_ $id
    $node log-target $T

    $ns_ at 0.0 "$node start-span"
}

