
#ifndef SPANAGENT_HH
#define SPANAGENT_HH

#include <vector>
#include <packet.h>
#include <stdarg.h>
#include <agent.h>
#include <trace.h>
#include <connector.h>
#include <packet.h>
#include <random.h>
#include <cmu/priqueue.h>
#include <cmu/cmu-trace.h>
#include <cmu/mac.h>
#include <cmu/mac-802_11.h>
#include <scheduler.h>
#include "hdr_span.h"
#include "spantable.hh"

#define SPAN_BCAST 		2.0   // broadcast interval
#define SPAN_ANNOUNCE_CHECK	0.5   // check announce interval
#define SPAN_WITHDRAW_CHECK	2.0   // check withdraw interval
#define SPAN_WITHDRAW_GRACE	8.0   // fwd after withdrawn
#define SPAN_BATTERY_LIFE	1800  // half an hour


class SPANAgent;

class SPANCheckAnnounceHandler : public Handler {
public: 
  SPANCheckAnnounceHandler(SPANAgent *r) : _agent(r) {}
  void handle(Event *e); 
private: 
  SPANAgent *_agent;
};

class SPANPeriodicBroadcastHandler : public Handler { 
public: 
  SPANPeriodicBroadcastHandler(SPANAgent *r) : _agent(r) {}
  void handle(Event *e); 
private: 
  SPANAgent *_agent;
};

class SPANAnnounceCoordinatorHandler : public Handler { 
public: 
  SPANAnnounceCoordinatorHandler(SPANAgent *r) : _agent(r) {}
  void handle(Event *e); 
private: 
  SPANAgent *_agent;
};

class SPANCheckWithdrawHandler : public Handler { 
public: 
  SPANCheckWithdrawHandler(SPANAgent *r) : _agent(r) {}
  void handle(Event *e); 
private: 
  SPANAgent *_agent;
};

class SPANTestGFHandler : public Handler {
public:
  SPANTestGFHandler(SPANAgent *r, unsigned d) : 
    _agent(r), _pkts(0), _dst(d) {}
  void handle(Event *e);
private:
  SPANAgent *_agent;
  int _pkts;
  unsigned _dst;
};

class GFRouter;

class SPANAgent : public Agent {
public:
  static const int _send_interval = 1;
  static const int _startup_jitter = 10; 
  static const double _max_energy_meter = SPAN_BATTERY_LIFE;

private:
  SPANPeriodicBroadcastHandler _periodic_broadcast_handler;
  SPANCheckAnnounceHandler _check_announce_handler;
  SPANCheckWithdrawHandler _check_withdraw_handler;
  SPANAnnounceCoordinatorHandler _announce_coordinator_handler;
  Event _periodic_broadcast_event;
  Event _check_withdraw_event;
  Event _check_announce_event;

  SPANNeighborTable _neighbors;
  int _self_handle;
  bool _is_coordinator;
  bool _asleep;

  bool _announce_pending;
  double _energy_meter;
  double _energy_trigger;
  double _last_bcast;
  unsigned _withdraw_meter;
  double _last_withdraw;

  nsaddr_t _my_id;
  int _off_mac;
  int _off_ll;
  int _off_cmn;
  int _off_ip;
  int _off_span;
  Trace *_logtarget;
  NsObject *_ll;
  LL *real_ll;
  PriQueue *_ifq;
  MobileNode *_mobile_node;
  Mac802_11 *_mac;

  unsigned _pkts_sent;
  unsigned _pkts_received;
  unsigned _pkts_hops;
  GFRouter *_gfrt;

  void process_span_hdr(Packet *p);
  void piggyback_span_hdr(Packet *p);
  static void tx_failure_cb(Packet *p, void *thunk);

public:
  SPANAgent();

  int id() const 		{ return _my_id; }
  int handle() const 		{ return _self_handle; }
  bool is_coordinator() const 	{ return _is_coordinator; }
  bool asleep() const		{ return _asleep; }
  double last_withdraw() const  { return _last_withdraw; }

  virtual int command(int argc, const char*const* argv);
  virtual void recv(Packet *, Handler *callback = 0);
  
  void broadcast_hello();
  void announce_coordinator();
  void withdraw_coordinator();

  // returns number of disjointed neighbors. if > 0, means we should 
  // become a coordinator
  int check_announce_coordinator();
  // returns true if current node should be withdraw as a coordinator
  bool check_withdraw_coordinator();

  void start();
  void wakeup();
  void sleep();
  
  void dump_table();
  void test_gf(nsaddr_t dst);

  void send_packet(Packet *p);
  void make_pkt(nsaddr_t dst, unsigned type);

  friend class GFRouter;  // drop()
  friend class SPANCheckAnnounceHandler;
};

#endif

