
#include "spanagent.hh"
#include "spantable.hh"
#define SECOND_HOP_REACHABILITY 1

int
SPANNeighborTable::share_coordinator
  (int a, int b, int ex, bool use_ex)
{
  if (a == b && (!use_ex || a != ex)) return a;
  int a_h = search_neighbor(a);
  int b_h = search_neighbor(b);
  if (a_h < 0 || b_h < 0) return -1;
  for (unsigned i=0; i<_neighbors[a_h].coordinators.size(); i++) {
    if (!_neighbors[a_h].coordinators[i].in_use || 
	_neighbors[a_h].coordinators[i].withdrawn) continue;
    int c_a = _neighbors[a_h].coordinators[i].coordinator_id;
    if (use_ex && c_a == ex) continue;
#if SECOND_HOP_REACHABILITY
    int ch_a = search_neighbor(c_a);
#endif
    for (unsigned j=0; j<_neighbors[b_h].coordinators.size(); j++) {
      if (!_neighbors[b_h].coordinators[i].in_use || 
	  _neighbors[b_h].coordinators[i].withdrawn) continue;
      int c_b = _neighbors[b_h].coordinators[j].coordinator_id;
      if (use_ex && c_b == ex) continue;
      if (c_a == c_b) return c_a;
#if SECOND_HOP_REACHABILITY
      if (ch_a >= 0) {
        for (unsigned k=0; k<_neighbors[ch_a].second_hops.size(); k++)
          if (c_b == _neighbors[ch_a].second_hops[k]) return c_b;
      }
#endif
    }
  } 
  return -1;
}

int
SPANNeighborTable::share_neighbor
  (int a, int b, int ex, bool use_ex)
{
  int a_h = search_neighbor(a);
  int b_h = search_neighbor(b);
  if (a_h < 0 || b_h < 0) return -1;
  if (a_h == b_h) return a;
  for (unsigned i=0; i<_neighbors[a_h].second_hops.size(); i++) {
    int s_a = _neighbors[a_h].second_hops[i];
    if (use_ex && s_a == ex) continue;
    for (unsigned j=0; j<_neighbors[b_h].second_hops.size(); j++) {
      int s_b = _neighbors[b_h].second_hops[j];
      if (use_ex && s_b == ex) continue;
      if (s_a == s_b) return s_a;
#if SECOND_HOP_REACHABILITY
      int sh_a = search_neighbor(s_a);
      if (sh_a >= 0) {
        for (unsigned k=0; k<_neighbors[sh_a].second_hops.size(); k++)
          if (s_b == _neighbors[sh_a].second_hops[k]) return s_b;
      }
#endif
    }
  } 
  return -1;
}

int
SPANNeighborTable::update_neighbor
  (int id, int hops, double nounce, bool is_c)
{
  int free = -1;
  int i = search_neighbor(id, free);
  if (i >= 0) {
    if (_neighbors[i].neighbor_nounce <= nounce) {
      _neighbors[i].is_coordinator = is_c;
      _neighbors[i].hops = hops;
      _neighbors[i].neighbor_nounce = nounce;
      return i;
    }
    return -1;
  }
  if (free >= 0) {
    _neighbors[free].in_use = 1;
    _neighbors[free].neighbor_id = id;
    _neighbors[free].is_coordinator = is_c;
    _neighbors[free].hops = hops;
    _neighbors[free].neighbor_nounce = nounce;
    _neighbors[free].coordinators.clear();
    return free;
  }
  else {
    SPANNeighborInfo ni;
    ni.in_use = 1;
    ni.neighbor_id = id;
    ni.is_coordinator = is_c;
    ni.hops = hops;
    ni.neighbor_nounce = nounce;
    ni.coordinators.clear();
    _neighbors.push_back(ni);
    return _neighbors.size()-1;
  }
}

int
SPANNeighborTable::add_coordinator(int node_h, int c)
{
  assert(node_h >= 0);
  assert((unsigned)node_h < _neighbors.size() && _neighbors[node_h].in_use);
  int free = -1;
  int j = search_coordinator(node_h, c, free);
  if (j >= 0) return j; 
  if (free >= 0) {
    _neighbors[node_h].coordinators[free].coordinator_id = c;
    _neighbors[node_h].coordinators[free].in_use = true;
    _neighbors[node_h].coordinators[free].withdrawn = false;
    return free; 
  }
  else {
    SPANCoordinatorInfo ci;
    ci.coordinator_id = c;
    ci.in_use = true;
    ci.withdrawn = false;
    _neighbors[node_h].coordinators.push_back(ci);
    return _neighbors[node_h].coordinators.size();
  }
}

int
SPANNeighborTable::expire_old_entries(int exclude, double now)
{
  SPANNeighborTableIterator *iter = iterator();
  while ((bool)(*iter)) {
    if (iter->neighbor_id() != exclude && 
	(now-iter->neighbor_nounce()) > _expire_period) {
      clear_coordinators(iter->handle());
      _neighbors[iter->handle()].in_use = false;
    }
    iter->next();
  }
  delete iter;
  return 0;
}

int
SPANNeighborTable::refresh_coordinators(int node)
{
  int node_h = search_neighbor(node);
  if (node_h < 0) return -1;
  SPANCoordinatorListIterator *citer = coordinator_iterator(node_h);
  while ((bool)(*citer)) {
    if (search_neighbor(citer->coordinator_id()) < 0) {
      _neighbors[node_h].coordinators[citer->handle()].in_use = false;
      _neighbors[node_h].coordinators[citer->handle()].withdrawn = true;
    }
    citer->next();
  }
  delete citer;
  return 0;
}

