#!/usr/bin/perl

if ($#ARGV < 3) {
    print stderr "usage: <num_nodes> <pkt_sz> <interval> <sim_time>\n";
    exit 2;
}
 
$num_nodes = $ARGV[0];
$pkt_sz = $ARGV[1];
$interval = $ARGV[2];
$end = $ARGV[3];

$max_jitter = 0.25 * $interval;

my $curr_cbr = 0;

sub send_one_pkt {
    # args: from-node to-node when pkt-size
    
    $src = @_[0];
    $dst = @_[1];
    $t = @_[2];
    $sz = @_[3];
    
    print "set cbr_($curr_cbr) [\$ns_ create-connection CBR \$node_($src) CBR \$node_($dst) 0]\n";
    print "\$cbr_($curr_cbr) set packetSize_ $sz\n";
    print "\$cbr_($curr_cbr) set interval_ 1000\n"; # does it matter, if only sending one pkt?
    print "\$cbr_($curr_cbr) set random_ 0\n";
    print "\$cbr_($curr_cbr) set maxpkts_ 1\n";
    print "\$ns_ at $t \"\$cbr_($curr_cbr) start\"\n";

    $curr_cbr++;
}


sub randomize {
    my $seed = $$;  #  $$ is pid
    my $buf;

    open(RAND_FILE,"/dev/urandom") or die "cannot open /dev/urandom";
    read(RAND_FILE,$buf, 4);
    $seed ^= $buf;

    close RAND_FILE;
    srand $seed;
}

sub int_rand {
    # produce random int r s.t. 0 <= r <= max-1
    $max = @_[0];
    return int(rand $max);
}


# every interval, each node sends one packet to some other random
# node.  the tx times are jittered so not everyone tries to send
# simultaneously.

print "# nodes: $num_nodes; pkt size: $pkt_sz; interval: $interval; end: $end\n";

randomize();
for ($now = $max_jitter; $now < $end; $now += $interval) {
    for ($i = 1; $i <= $num_nodes; $i++) {
	$jitter = rand $max_jitter;
	$when = $now + $jitter - $max_jitter/2;
	$dest = $i;
	do {
	    $dest = 1 + int_rand($num_nodes);
	} while ($dest == $i);
	send_one_pkt($i, $dest, $when, $pkt_sz);
    }
}
