#!/usr/bin/perl

if ($#ARGV < 3) {
    print stderr "usage: <num_nodes> <pkt_sz> <avg-node-rate> <sim_time>\n";
    exit 2;
}
 
$num_nodes = $ARGV[0];
$pkt_sz = $ARGV[1];
$avg_rate = $ARGV[2];
$end = $ARGV[3];

$avg_interval = 1 / $avg_rate;

my $curr_cbr = 0;

sub send_one_pkt {
    # args: from-node to-node when pkt-size
    
    $src = @_[0];
    $dst = @_[1];
    $t = @_[2];
    $sz = @_[3];
    
    print "set cbr_($curr_cbr) [\$ns_ create-connection CBR \$node_($src) CBR \$node_($dst) 0]\n";
    print "\$cbr_($curr_cbr) set packetSize_ $sz\n";
    print "\$cbr_($curr_cbr) set interval_ 1000\n"; # does it matter, if only sending one pkt?
    print "\$cbr_($curr_cbr) set random_ 0\n";
    print "\$cbr_($curr_cbr) set maxpkts_ 1\n";
    print "\$ns_ at $t \"\$cbr_($curr_cbr) start\"\n";

    $curr_cbr++;
}


sub randomize {
    my $seed = $$;  #  $$ is pid
    my $buf;

    open(RAND_FILE,"/dev/urandom") or die "cannot open /dev/urandom";
    read(RAND_FILE,$buf, 4);
    $seed ^= $buf;

    close RAND_FILE;
    srand $seed;
}

sub int_rand {
    # produce random int r s.t. 0 <= r <= max-1
    $max = @_[0];
    return int(rand $max);
}




# for each node, send packets with uniformly distributed interarrival
# times in [0, 2 * avg_rate].  This should produce an average rate of
# avg_rate for each node.  But is the sum of all nodes sending during
# a fixed interval smooth across al intervals?

print "# nodes: $num_nodes; pkt size: $pkt_sz; avg rate: $avg_rate; end: $end\n";

randomize();

# slight weirdness: output all pkts originated by each node before
# doing the packets for another node e.g. cbrs are not in time order.
# ns scheduler can handle this okay.
for ($i = 1; $i <= $num_nodes; $i++) {
    $now = rand (2 * $avg_interval);
    for ( ; $now <= $end; $now += rand (2 * $avg_interval)) {
	$dest = $i;
	do {
	    $dest = 1 + int_rand($num_nodes);
	} while ($dest == $i);
	send_one_pkt($i, $dest, $now, $pkt_sz);
    }
}
